==> bigloo2.0a.tar.gz: bigloo2.0a/runtime/Ieee/equiv.scm <==
;*---------------------------------------------------------------------*/
;*   A pratical implementation for the Scheme programming language     */
;*                                                                     */
;*                                    ,--^,                            */
;*                              _ ___/ /|/                             */
;*                          ,;'( )__, ) '                              */
;*                         ;;  //   L__.                               */
;*                         '   \\   /  '                               */
;*                              ^   ^                                  */
;*                                                                     */
;*   Copyright (c) 1992-1999 Manuel Serrano                            */
;*                                                                     */
;*     Bug descriptions, use reports, comments or suggestions are      */
;*     welcome. Send them to                                           */
;*       bigloo-request@kaolin.unice.fr                                */
;*       http://kaolin.unice.fr/bigloo                                 */
;*                                                                     */
;*   This program is free software; you can redistribute it            */
;*   and/or modify it under the terms of the GNU General Public        */
;*   License as published by the Free Software Foundation; either      */
;*   version 2 of the License, or (at your option) any later version.  */
;*                                                                     */
;*   This program is distributed in the hope that it will be useful,   */
;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
;*   GNU General Public License for more details.                      */
;*                                                                     */
;*   You should have received a copy of the GNU General Public         */
;*   License along with this program; if not, write to the Free        */
;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
;*   MA 02111-1307, USA.                                               */
;*---------------------------------------------------------------------*/
;*=====================================================================*/
;*    serrano/prgm/project/bigloo/runtime/Ieee/equiv.scm               */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Fri Jan 20 09:57:55 1995                          */
;*    Last change :  Mon Mar  2 07:10:18 1998 (serrano)                */
;*    -------------------------------------------------------------    */
;*    6.2. Equivalence predicates (page 13, r4)                        */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __r4_equivalence_6_2
   
   (import  (__error                   "Llib/error.scm"))
   
   (use     (__type                    "Llib/type.scm")
	    (__bigloo                  "Llib/bigloo.scm")
	    (__tvector                 "Llib/tvector.scm")
	    (__structure               "Llib/struct.scm")
	    (__object                  "Llib/object.scm")
	    (__ucs2                    "Llib/ucs2.scm")
	    (__unicode                 "Llib/unicode.scm")
	    
	    (__r4_numbers_6_5          "Ieee/number.scm")
	    (__r4_numbers_6_5_fixnum   "Ieee/fixnum.scm")
	    (__r4_booleans_6_1         "Ieee/boolean.scm")
	    (__r4_characters_6_6       "Ieee/char.scm")
	    (__r4_pairs_and_lists_6_3  "Ieee/pair-list.scm")
	    (__r4_vectors_6_8          "Ieee/vector.scm")
	    (__r4_numbers_6_5_flonum   "Ieee/flonum.scm")
	    (__r4_symbols_6_4          "Ieee/symbol.scm")
	    (__r4_strings_6_7          "Ieee/string.scm")
	    (__r4_control_features_6_9 "Ieee/control.scm")
	    
	    (__evenv                   "Eval/evenv.scm"))
   
   (foreign (infix macro bool c-eq? (obj obj) "=="))
   
   (export  (eqv?::bool ::obj ::obj)
	    (inline eq?::bool  ::obj ::obj) 
	    (equal?::bool      ::obj ::obj))

   (pragma  (eqv? side-effect-free)
	    (eq? side-effect-free)
	    (c-eq? side-effect-free no-cfa-top)
	    (equal? side-effect-free no-cfa-top)))

;*---------------------------------------------------------------------*/
;*    eq? ...                                                          */
;*---------------------------------------------------------------------*/
(define-inline (eq? obj1 obj2)
   (c-eq? obj1 obj2))

;*---------------------------------------------------------------------*/
;*    eqv? ...                                                         */
;*---------------------------------------------------------------------*/
(define (eqv? obj1 obj2)
   (cond
      ((exact? obj1)
       (and (exact? obj2)
	    (= obj1 obj2)))
      ((inexact? obj1)
       (and (inexact? obj2)
	    (= obj1 obj2)))
      ((symbol? obj1)
       (and (symbol? obj2)
	    (string=? (symbol->string obj1)
		      (symbol->string obj2))))
      (else
       (eq? obj1 obj2))))

;*---------------------------------------------------------------------*/
;*    equal? ...                                                       */
;*---------------------------------------------------------------------*/
(define (equal? obj1 obj2)
   (cond
      ((c-eq? obj1 obj2)
       #t)
      ((fixnum? obj1)
       #f)
      ((symbol? obj1)
       #f)
      ((ucs2? obj1)
       #f)
      ((pair? obj1)
       (and (pair? obj2) (equal? (car obj1) (car obj2))
	    (equal? (cdr obj1) (cdr obj2))))
      ((vector? obj1)
       (let ((lobj1 (vector-length obj1)))
	  (and (vector? obj2)
	       (=fx (vector-length obj2) lobj1)
	       (=fx (vector-tag obj1) (vector-tag obj2))
	       (let test ((i (-fx lobj1 1)))
		  (or (=fx i -1)
		      (and (equal? (vector-ref-ur obj1 i)
				   (vector-ref-ur obj2 i))
			   (test (-fx i 1))))))))
      ((string? obj1)
       (and (string? obj2) (string=? obj1 obj2)))
      ((c-flonum? obj1)
       (and (c-flonum? obj2)
	    (=fl obj1 obj2)))
      ((c-struct? obj1)
       (let ((lobj1 (struct-length obj1)))
	  (and (struct? obj2)
	       (=fx (struct-length obj2) lobj1)
	       (let test ((i (-fx lobj1 1)))
		  (or (=fx i -1)
		      (and (equal? (struct-ref obj1 i) (struct-ref obj2 i))
			   (test (-fx i 1))))))))
      ((cell? obj1)
       (and (cell? obj2)
	    (equal? (cell-ref obj1) (cell-ref obj2))))
      ((number? obj1)
       (and (number? obj2)
	    (= obj1 obj2)))
      ((object? obj1)
       (and (object? obj2)
	    (object-equal? obj1 obj2)))
      ((ucs2-string? obj1)
       (and (ucs2-string? obj2) (ucs2-string=? obj1 obj2)))
      (else
       #f)))

==> bigloo_2.4b.orig.tar.gz: bigloo2.4b/runtime/Ieee/equiv.scm <==
;*=====================================================================*/
;*    serrano/prgm/project/bigloo/runtime/Ieee/equiv.scm               */
;*    -------------------------------------------------------------    */
;*    Author      :  Manuel Serrano                                    */
;*    Creation    :  Fri Jan 20 09:57:55 1995                          */
;*    Last change :  Tue Jul  3 16:41:36 2001 (serrano)                */
;*    -------------------------------------------------------------    */
;*    6.2. Equivalence predicates (page 13, r4)                        */
;*=====================================================================*/

;*---------------------------------------------------------------------*/
;*    The module                                                       */
;*---------------------------------------------------------------------*/
(module __r4_equivalence_6_2
   
   (import  __error)
   
   (use     __type
	    __bigloo
	    __tvector
	    __structure
	    __object
	    __ucs2
	    __unicode
	    __custom
	    __foreign
	    __os

	    __r4_numbers_6_5
	    __r4_numbers_6_5_fixnum
	    __r4_booleans_6_1
	    __r4_characters_6_6
	    __r4_pairs_and_lists_6_3
	    __r4_vectors_6_8
	    __r4_numbers_6_5_flonum
	    __r4_symbols_6_4
	    __r4_strings_6_7
	    __r4_control_features_6_9
	    
	    __evenv)
   
   (extern  (infix macro c-eq?::bool (::obj ::obj) "=="))
   
   (java    (class foreign
	       (method static c-eq?::bool (::obj ::obj) "EQ")
	       (method static c-boxed-eq?::bool (::obj ::obj) "BOXED_EQ")))
   
   (export  (eqv?::bool ::obj ::obj)
	    (inline eq?::bool  ::obj ::obj)
	    (equal?::bool ::obj ::obj))
   
   (pragma  (eqv? side-effect-free nesting)
	    (eq? side-effect-free nesting)
	    (c-eq? side-effect-free no-cfa-top nesting)
	    (equal? side-effect-free no-cfa-top nesting)))

;*---------------------------------------------------------------------*/
;*    eq? ...                                                          */
;*---------------------------------------------------------------------*/
(define-inline (eq? obj1 obj2)
   (c-eq? obj1 obj2))

;*---------------------------------------------------------------------*/
;*    eqv? ...                                                         */
;*---------------------------------------------------------------------*/
(define (eqv? obj1 obj2)
   (cond
      ((exact? obj1)
       (and (exact? obj2)
	    (= obj1 obj2)))
      ((inexact? obj1)
       (and (inexact? obj2)
	    (= obj1 obj2)))
      ((symbol? obj1)
       (and (symbol? obj2)
	    (string=? (symbol->string obj1)
		      (symbol->string obj2))))
      ((foreign? obj1)
       (and (foreign? obj2)
	    (foreign-eq? obj1 obj2)))
      (else
       (eq? obj1 obj2))))

;*---------------------------------------------------------------------*/
;*    equal? ...                                                       */
;*---------------------------------------------------------------------*/
(define (equal? obj1 obj2)
   (cond
      ((c-eq? obj1 obj2)
       #t)
      ((fixnum? obj1)
       (if (fixnum? obj2)
	   (=fx obj1 obj2)
	   #f))
      ((symbol? obj1)
       #f)
      ((pair? obj1)
       (and (pair? obj2) (equal? (car obj1) (car obj2))
	    (equal? (cdr obj1) (cdr obj2))))
      ((vector? obj1)
       (let ((lobj1 (vector-length obj1)))
	  (and (vector? obj2)
	       (=fx (vector-length obj2) lobj1)
	       (=fx (vector-tag obj1) (vector-tag obj2))
	       (let test ((i 0))
		  (or (=fx i lobj1)
		      (and (equal? (vector-ref-ur obj1 i)
				   (vector-ref-ur obj2 i))
			   (test (+fx i 1))))))))
      ((string? obj1)
       (and (string? obj2) (string=? obj1 obj2)))
      ((c-flonum? obj1)
       (and (c-flonum? obj2)
	    (=fl obj1 obj2)))
      ((c-struct? obj1)
       (let ((lobj1 (struct-length obj1)))
	  (and (struct? obj2)
	       (=fx (struct-length obj2) lobj1)
	       (let test ((i 0))
		  (or (=fx i lobj1)
		      (and (equal? (struct-ref obj1 i) (struct-ref obj2 i))
			   (test (+fx i 1))))))))
      ((cell? obj1)
       (and (cell? obj2)
	    (equal? (cell-ref obj1) (cell-ref obj2))))
      ((number? obj1)
       (and (number? obj2)
	    (= obj1 obj2)))
      ((object? obj1)
       (and (object? obj2)
	    (object-equal? obj1 obj2)))
      ((ucs2-string? obj1)
       (and (ucs2-string? obj2) (ucs2-string=? obj1 obj2)))
      ((custom? obj1)
       (and (custom? obj2) (custom-equal? obj1 obj2)))
      ((foreign? obj1)
       (and (foreign? obj2)
	    (foreign-eq? obj1 obj2)))
      ((ucs2? obj1)
       (and (ucs2? obj2) (ucs2=? obj1 obj2)))
      (else
       #f)))
